;;; The Module ADT, implemented as a Scheme library.

(define-library (module)

  (export makeModule
          moduleName
          moduleUses
          modificationTime
          compilationTime
          compilationMode)

  (import (scheme base))

  ;; makeModule : String ListOfString Int Int String -> Module
  ;; GIVEN: the name of a module,
  ;;     a list of the names of all modules it uses
  ;;     a non-negative integer giving the time at which the module's
  ;;     source code was last modified (as the number of seconds that
  ;;     have elapsed since midnight on 1 January 1970),
  ;;     an integer giving the time at which the module was last compiled
  ;;     (or -1 if the module has not been compiled),
  ;;     and a string naming the mode in which the module was last compiled
  ;;     (which is meaningless if the module has not been compiled)
  ;; RETURNS: a description encapsulating the given information
  ;;
  ;; moduleName : Module -> String
  ;; moduleUses : Module -> ListOfString
  ;; modificationTime : Module -> Int
  ;; compilationTime : Module -> Int
  ;; compilationMode : Module -> String
  ;;
  ;; GIVEN: the description of a module
  ;; RETURNS: the appropriate part of that description
  ;; EXAMPLES:
  ;; moduleName (makeModule ("Foo", ["Baz"], 1505109465, 1504097449, "LP64"))
  ;;     =>  "Foo"
  ;; moduleUses (makeModule ("Foo", ["Baz"], 1505109465, 1504097449, "LP64"))
  ;;     =>  ["Baz"]
  ;; modificationTime (makeModule ("Foo", ["Baz"], 1505109465, 1504097449, "LP64"))
  ;;     =>  1505109465
  ;; compilationTime (makeModule ("Foo", ["Baz"], 1505109465, 1504097449, "LP64"))
  ;;     =>  1504097449
  ;; compilationMode (makeModule ("Foo", ["Baz"], 1505109465, 1504097449, "LP64"))
  ;;     =>  "LP64"

  (begin

   (define-record-type :Module
     (makeModule name uses mtime ctime cmode)
     module?
     (name  moduleName)
     (uses  moduleUses)
     (mtime modificationTime)
     (ctime compilationTime)
     (cmode compilationMode))))
